# Tesses.VirtualFilesystem

A library for virtual filesystems in .NET

# Nuget
Tesses.VirtualFilesystem.Base

[![Downloads](https://img.shields.io/nuget/dt/Tesses.VirtualFilesystem.Base.svg)](https://nuget.org/packages/Tesses.VirtualFilesystem.Base)
[![Version](https://img.shields.io/nuget/v/Tesses.VirtualFilesystem.Base.svg)](https://nuget.org/packages/Tesses.VirtualFilesystem.Base)

Tesses.VirtualFilesystem

[![Downloads](https://img.shields.io/nuget/dt/Tesses.VirtualFilesystem.svg)](https://nuget.org/packages/Tesses.VirtualFilesystem)
[![Version](https://img.shields.io/nuget/v/Tesses.VirtualFilesystem.svg)](https://nuget.org/packages/Tesses.VirtualFilesystem)

Tesses.VirtualFilesystem.Local

[![Downloads](https://img.shields.io/nuget/dt/Tesses.VirtualFilesystem.Local.svg)](https://nuget.org/packages/Tesses.VirtualFilesystem.Local)
[![Version](https://img.shields.io/nuget/v/Tesses.VirtualFilesystem.Local.svg)](https://nuget.org/packages/Tesses.VirtualFilesystem.Local)

Tesses.VirtualFilesystem.Zio

[![Downloads](https://img.shields.io/nuget/dt/Tesses.VirtualFilesystem.Zio.svg)](https://nuget.org/packages/Tesses.VirtualFilesystem.Zio)
[![Version](https://img.shields.io/nuget/v/Tesses.VirtualFilesystem.Zio.svg)](https://nuget.org/packages/Tesses.VirtualFilesystem.Zio)

Tesses.VirtualFilesystem.Extensions

[![Downloads](https://img.shields.io/nuget/dt/Tesses.VirtualFilesystem.Extensions.svg)](https://nuget.org/packages/Tesses.VirtualFilesystem.Extensions)
[![Version](https://img.shields.io/nuget/v/Tesses.VirtualFilesystem.Extensions.svg)](https://nuget.org/packages/Tesses.VirtualFilesystem.Extensions)

# License

![GPL3-only](https://www.gnu.org/graphics/gplv3-with-text-136x68.png) 
Licensed under [GPL3](https://www.gnu.org/licenses/)

# How to use (way 1 uses LocalFileSystem)
```csharp
using Tesses.VirtualFilesystem;
using Tesses.VirtualFilesystem.Filesystems;
using Tesses.VirtualFilesystem.Extensions;

LocalFileSystem fileSystem = new LocalFileSystem();
//Special.Home is $HOME on linux (usually /home/username) or %UserProfile% on Windows (usually C:\Users\username)
foreach(var item in fileSystem.EnumerateDirectories(Special.Home))
{
    Console.WriteLine(item.Name); //this is name
    //Console.WriteLine(item.Path) //this is entire path
}
fileSystem.WriteAllText(Special.CurDir / "file.txt","Some Text"); //write to file in current directory
fileSystem.CreateDirectory(Special.CurDir / "dir1/dir2/dir3" / "dir4"); //shows how you can use normal paths too within special ones

//enumerates directories in linux home dir (not cross platform) 
foreach(var item in fileSystem.EnumerateDirectories("/home")) 
{
    Console.WriteLine(item.Name);
}

fileSystem.CreateSymlink("fileToLinkTo","symlinkFile");
fileSystem.ReadSymlink("symlinkFile"); //should be equal to Special.CurDir / "fileToLinkTo" in this instance

fileSystem.CreateHardlink("originalFileName","newFileName"); //for people who dont know what hardlinks are, a simple explanation is a hard link is giving a file another name (not copying) and can only be done to files and on same real filesystem

fileSystem.DirectoryExists(Special.Home); //should be true if directory exists
```
# How to use (way 2 uses LocalFileSystem)
```csharp
using Tesses.VirtualFilesystem;
using Tesses.VirtualFilesystem.Filesystems;
using Tesses.VirtualFilesystem.Extensions;

var res=fileSystem.OpenDirectory(Special.Home);
foreach(var item in res)
{
    if(item.IsDirectoryPointer)
    {
        var dir=item.AsDirectoryPointer();
        //dir.EnumerateDirectories(); //enumerate directories IEnumerable<DirectoryPointer>
        Console.WriteLine($"DIR: {item.Name}");
    }
    if(item.IsFilePointer)
    {
        var file=item.AsFilePointer();
        //file.Delete(); //deletes file
        Console.WriteLine($"FILE: {item.Name}");
    }
    if(item.IsSymlinkPointer)
    {
        var link = item.AsSymlinkPointer();
        //do something with symlink
        //link.Target = Special.Music; //sets link to Music folder
        Console.WriteLine($"LINK: {item.Name}");
    }
}

```

# How to implement

Inherit from SyncFileSystem or AsyncFileSystem

Look in the source for Tesses.VirtualFilesystem.Local for help with SyncFileSystem (I might add documentation for this later)